# Copyright (c) 2008-2009 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: interfaces.py 16794 2009-07-28 09:02:48Z ctheune $

import re

import grok
import zc.sourcefactory.basic
import zc.sourcefactory.contextual
import zope.interface
import zope.schema
import zope.component
import zope.app.component.hooks


class IResource(zope.interface.Interface):

    title = zope.interface.Attribute('A human-readable title for the resource.')


class IDirectory(zope.interface.Interface):
    """The directory."""

    paltry = zope.schema.ASCIILine(
        title=u'Location of paltry directory')

    locations = zope.interface.Attribute('All locations')
    vlan_classes = zope.interface.Attribute('All VLAN classes')


class IResourceGroup(zope.interface.Interface):
    """A group of resources."""

    name = zope.schema.ASCIILine(title=u'Name')
    nodes = zope.interface.Attribute(
        "All nodes that belong to this resource group.")


class IPaltryDirectory(zope.interface.Interface):
    """Model-level API to access the YAML shim."""

    def get_password(service):
        pass

    def list_people():
        pass

    def list_groups():
        pass

    def list_locations():
        pass

    def lookup_location(name):
        pass

    def lookup_networks(location, af=''):
        pass

    def list_networks():
        pass

    def lookup_interfaces(nodename, af_filter=''):
        pass

    def lookup_addresses(nodename, vlan_filter='', af_filter=''):
        pass


class ILocation(zope.interface.Interface):

    title = zope.schema.TextLine(title=u'Title')
    abbreviation = zope.schema.ASCIILine(title=u'Short name / Identifier')

    vlans = zope.interface.Attribute('The VLANs of this location.')


class IVLANClass(zope.interface.Interface):

    title = zope.schema.TextLine(title=u'Titel')
    short = zope.schema.ASCIILine(title=u'Kurzname')
    description = zope.schema.TextLine(title=u'Description')
    number = zope.schema.Int(title=u'Number')
    color = zope.schema.ASCIILine(title=u'Color (hex)')


class IVLAN(zope.interface.Interface):
    """A network segment in a location."""

    class_ = zope.interface.Attribute('The VLAN class this vlan belongs to.')

    networks = zope.interface.Attribute(
        'The networks reachable on this VLAN.')


class INetwork(zope.interface.Interface):
    """An IP network."""

    cidr = zope.schema.ASCIILine(
        title=u'Network',
        description=u'Please use a CIDR representation of the network.')

    def allocate(index, resource):
        """Allocate an address for the given resource.

        If the address has already been allocated a KeyError will be raised.

        If the index is `None` then an address will be automatically picked.
        """


class IAddress(zope.interface.Interface):
    """An IP address."""

    network = zope.interface.Attribute('The network this address belongs to.')
    resource = zope.interface.Attribute(
        'The resource this address is being used by.')
    ip = zope.interface.Attribute('The IP this address represents.')
    index = zope.interface.Attribute(
        'The index the address has in its network')


class NICNetworks(zc.sourcefactory.contextual.BasicContextualSourceFactory):

    def getValues(self, nic):
        for network in nic.vlan.networks:
            yield network

    def getTitle(self, nic, network):
        return network.cidr


class NICVLANs(zc.sourcefactory.contextual.BasicContextualSourceFactory):

    def getValues(self, nic):
        if INode.providedBy(nic):
            node = nic
        else:
            node = nic.node
        return node.location.vlans

    def getTitle(self, nic, vlan):
        return vlan.class_.title


class INIC(zope.interface.Interface):
    """A network interface card."""

    mac = zope.schema.ASCIILine(title=u'MAC address')
    vlan = zope.schema.Choice(source=NICVLANs())
    addresses = zope.interface.Attribute(
        u'Addresses associated with this NIC.')


class Locations(zc.sourcefactory.contextual.BasicContextualSourceFactory):

    def getValues(self, context):
        directory = zope.app.component.hooks.getSite()
        return directory.locations

    def getTitle(self, context, location):
        return location.title


class PuppetClasses(zc.sourcefactory.basic.BasicSourceFactory):

    def getValues(self):
        return ["role::customerproject",
                "role::directory_server",
                "role::generic",
                "role::kvm_host",
                "role::lock_server",
                "role::nagios",
                "role::nameserver",
                "role::portage_binhost",
                "role::puppetmaster",
                "role::router",
                "role::storageserver",
                "role::switch",
                "sys_block::megaraid"]


class INode(zope.interface.Interface):

    class_ = zope.interface.Attribute('Type of node: virtual or physical')
    nics = zope.interface.Attribute('List of NICs for this node.')
    resource_group = zope.interface.Attribute(
        u'The resource group this node belongs to')

    name = zope.schema.ASCIILine(title=u'Hostname')
    # XXX make unique, generate automatically
    global_id = zope.schema.Int(title=u'Global node ID',
                                min=4100, max=65535)
    location = zope.schema.Choice(
        title=u'Location',
        source=Locations())

    puppet_classes = zope.schema.Set(
        title=u'Puppet-Klassen',
        value_type=zope.schema.Choice(source=PuppetClasses()))

    environment = zope.schema.Choice(title=u'Environment',
                                     values=['dev', 'staging', 'production'])
    contacts = zope.schema.TextLine(
        title=u'Kontaktpersonen')

    # XXX Password can be automatically generated
    directory_password = zope.interface.Attribute(
        'The directory API password for this node')
    directory_ring = zope.schema.Int(
        title=u'Access level (ring) for directory API')


class KVMHosts(zc.sourcefactory.contextual.BasicContextualSourceFactory):

    def getValues(self, context):
        yield None
        directory = zope.app.component.hooks.getSite()
        for node in directory.nodes:
            if 'role::kvm_host' in node.puppet_classes:
                yield node

    def getTitle(self, context, node):
        if node is None:
            return u'--disabled--'
        return node.name

    def getToken(self, context, node):
        if node is None:
            return '.none'
        return super(KVMHosts, self).getToken(context, node)


class IVirtualMachine(INode):

    location = zope.interface.Attribute(
        'Override schema-field of INode. Location is derived from kvm_host')

    # XXX This is computed and storage-implementation specific. Do this on client?
    rootdev = zope.interface.Attribute('Name of the root device')

    memory = zope.schema.Int(title=u'Memory in MiB')

    # XXX setting the kvm host determines the location automatically. Once a
    # location has been set the host can not be changed to a different
    # location as the data would be on an unreachable storage server
    # otherwise.
    kvm_host = zope.schema.Choice(title=u'KVM host', source=KVMHosts())


class INodes(zope.interface.Interface):
      """A dictionary-like collection of nodes."""


class IPerson(zope.interface.Interface):

    cn = zope.schema.TextLine(title=u'Name')
    uid = zope.schema.ASCIILine(title=u'Login')
    mail = zope.schema.TextLine(title=u'E-mail address')
    pager = zope.schema.TextLine(title=u'Pager')


class IGroup(zope.interface.Interface):

    name = zope.schema.ASCIILine(title=u'Group name')
    persons = zope.interface.Attribute('List of persons')


class IPassword(zope.interface.Interface):

    clear_text = zope.schema.TextLine(
        title=u'The clear text representation of the password')


class IConsistencyCheck(zope.interface.Interface):
    """Provides a consistency check for a context.

    Produces a list of errors when inconsistencies are found.

    Register objects as subscribers to this interface to be picked up by the
    global consistency check mechanisms.

    """

    errors = zope.interface.Attribute('The list of errors.')
